@@ -1,11 +1,11 @@
 /*
-  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization
+  Copyright @ 1999 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
 
   You may not use this file except in compliance with the License.  You may
   obtain a copy of the License at
 
-    https://imagemagick.org/script/license.php
+    https://imagemagick.org/license/
 
   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
@@ -22,9 +22,8 @@
 extern "C" {
 #endif
 
-#ifndef StartMetaPixelChannel
-  #define StartMetaPixelChannel ((PixelChannel) (CompositeMaskPixelChannel+1))
-#endif
+extern MagickPrivate MagickBooleanType
+  ResetPixelChannelMap(Image *,ExceptionInfo *);
 
 #if defined(__cplusplus) || defined(c_plusplus)
 }
