@@ -18,13 +18,13 @@
 %                                August 2007                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright @ 2007 ImageMagick Studio LLC, a non-profit organization         %
+%  Copyright @ 1999 ImageMagick Studio LLC, a non-profit organization         %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    https://imagemagick.org/script/license.php                               %
+%    https://imagemagick.org/license/                                         %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -598,25 +598,31 @@ MagickExport MagickBooleanType ResamplePixelColor(
       weight = 0.0;
 #if FILTER_LUT
       /* Note that the ellipse has been pre-scaled so F = WLUT_WIDTH */
-      if ((Q >= 0.0) && ((int) Q < WLUT_WIDTH)) {
-        weight = resample_filter->filter_lut[(int)Q];
+      if (((int) Q >= 0) && ((int) Q < WLUT_WIDTH)) {
+        weight = resample_filter->filter_lut[(int) Q];
 #else
       /* Note that the ellipse has been pre-scaled so F = support^2 */
-      if ((Q >= 0.0) && (Q < (double)resample_filter->F)) {
+      if ((Q >= 0.0) && (Q < resample_filter->F)) {
         weight = GetResizeFilterWeight(resample_filter->filter_def,
              sqrt(Q));    /* a SquareRoot!  Arrggghhhhh... */
 #endif
 
-        pixel->alpha  += weight*GetPixelAlpha(resample_filter->image,pixels);
+        pixel->alpha+=weight*(double)
+          GetPixelAlpha(resample_filter->image,pixels);
         divisor_m += weight;
 
         if (pixel->alpha_trait != UndefinedPixelTrait)
-          weight *= QuantumScale*((double) GetPixelAlpha(resample_filter->image,pixels));
-        pixel->red   += weight*GetPixelRed(resample_filter->image,pixels);
-        pixel->green += weight*GetPixelGreen(resample_filter->image,pixels);
-        pixel->blue  += weight*GetPixelBlue(resample_filter->image,pixels);
+          weight*=QuantumScale*((double)
+            GetPixelAlpha(resample_filter->image,pixels));
+        pixel->red+=weight*(double)
+          GetPixelRed(resample_filter->image,pixels);
+        pixel->green+=weight*(double)
+          GetPixelGreen(resample_filter->image,pixels);
+        pixel->blue+=weight*(double)
+          GetPixelBlue(resample_filter->image,pixels);
         if (pixel->colorspace == CMYKColorspace)
-          pixel->black += weight*GetPixelBlack(resample_filter->image,pixels);
+          pixel->black+=weight*(double)
+            GetPixelBlack(resample_filter->image,pixels);
         divisor_c += weight;
 
         hit++;
@@ -636,7 +642,7 @@ MagickExport MagickBooleanType ResamplePixelColor(
 #else
       }
 #endif
-      pixels+=GetPixelChannels(resample_filter->image);
+      pixels+=(ptrdiff_t) GetPixelChannels(resample_filter->image);
       Q += DQ;
       DQ += DDQ;
     }
@@ -1201,10 +1207,10 @@ MagickExport void ScaleResampleFilter(ResampleFilter *resample_filter,
   { double scale;
 #if FILTER_LUT
     /* scale so that F = WLUT_WIDTH; -- hardcoded */
-    scale=(double) WLUT_WIDTH*PerceptibleReciprocal(F);
+    scale=(double) WLUT_WIDTH*MagickSafeReciprocal(F);
 #else
     /* scale so that F = resample_filter->F (support^2) */
-    scale=resample_filter->F*PerceptibleReciprocal(F);
+    scale=resample_filter->F*MagickSafeReciprocal(F);
 #endif
     resample_filter->A = A*scale;
     resample_filter->B = B*scale;
