@@ -23,7 +23,7 @@
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    https://imagemagick.org/script/license.php                               %
+%    https://imagemagick.org/license/                                         %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -101,7 +101,7 @@ MagickExport const char *GetMagickCopyright(void)
 MagickExport const char *GetMagickDelegates(void)
 {
   static const char
-    *delegates = ""
+    *delegates=""
 #if defined(MAGICKCORE_AUTOTRACE_DELEGATE)
   " autotrace"
 #endif
@@ -199,6 +199,9 @@ MagickExport const char *GetMagickDelegates(void)
 #if defined(MAGICKCORE_TIFF_DELEGATE)
   " tiff"
 #endif
+#if defined(MAGICKCORE_UHDR_DELEGATE)
+  " uhdr"
+#endif
 #if defined(MAGICKCORE_WEBP_DELEGATE)
   " webp"
 #endif
@@ -221,6 +224,8 @@ MagickExport const char *GetMagickDelegates(void)
   " zstd"
 #endif
   ;
+  if (*delegates == '\0')
+    return(delegates);
   return(delegates+1);
 }
 
@@ -262,7 +267,7 @@ MagickExport const char *GetMagickFeatures(void)
 #if defined(MAGICKCORE_HDRI_SUPPORT)
   "HDRI "
 #endif
-#if defined(MAGICKCORE_BUILD_MODULES) || defined(_DLL)
+#if defined(MAGICKCORE_BUILD_MODULES)
   "Modules "
 #endif
 #if defined(MAGICKCORE_OPENCL_SUPPORT)
@@ -498,7 +503,7 @@ MagickExport const char *GetMagickReleaseDate(void)
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
 %  GetMagickSignature() returns a signature that uniquely encodes the
-%  MagickCore library version, quantum depth, HDRI status, OS word size, 
+%  MagickCore library version, quantum depth, HDRI status, OS word size,
 %  channel type, and endianness.
 %
 %  The format of the GetMagickSignature method is:
@@ -569,20 +574,20 @@ MagickExport unsigned int GetMagickSignature(const StringInfo *nonce)
   p=GetStringInfoDatum(version);
   signature=MAGICKCORE_QUANTUM_DEPTH;
   (void) memcpy(p,&signature,sizeof(signature));
-  p+=sizeof(signature);
+  p+=(ptrdiff_t) sizeof(signature);
   signature=MAGICKCORE_HDRI_ENABLE;
   (void) memcpy(p,&signature,sizeof(signature));
-  p+=sizeof(signature);
+  p+=(ptrdiff_t) sizeof(signature);
   signature=MagickLibInterface;
   (void) memcpy(p,&signature,sizeof(signature));
-  p+=sizeof(signature);
+  p+=(ptrdiff_t) sizeof(signature);
   signature=1;  /* endianness */
   (void) memcpy(p,&signature,sizeof(signature));
-  p+=sizeof(signature);
+  p+=(ptrdiff_t) sizeof(signature);
 #if defined(MAGICKCORE_64BIT_CHANNEL_MASK_SUPPORT)
   signature=sizeof(ChannelType);
   (void) memcpy(p,&signature,sizeof(signature));
-  p+=sizeof(signature);
+  p+=(ptrdiff_t) sizeof(signature);
 #endif
   SetStringInfoLength(version,(size_t) (p-GetStringInfoDatum(version)));
   if (nonce != (const StringInfo *) NULL)
@@ -658,12 +663,12 @@ MagickExport void ListMagickVersion(FILE *file)
 #if defined(MAGICKCORE_MSC_VER)
   (void) FormatLocaleFile(file,"Compiler: Visual Studio %d (%d)\n",
     MAGICKCORE_MSC_VER,_MSC_FULL_VER);
-#elif defined(__GNUC__)
-  (void) FormatLocaleFile(file,"Compiler: gcc (%d.%d)\n",__GNUC__,
-    __GNUC_MINOR__);
 #elif defined(__clang__)
   (void) FormatLocaleFile(file,"Compiler: clang (%d.%d.%d)\n",__clang_major__,
     __clang_minor__,__clang_patchlevel__);
+#elif defined(__GNUC__)
+  (void) FormatLocaleFile(file,"Compiler: gcc (%d.%d)\n",__GNUC__,
+    __GNUC_MINOR__);
 #elif defined(__MINGW32_MAJOR_VERSION)
   (void) FormatLocaleFile(file,"Compiler: MinGW (%d.%d)\n",
     __MINGW32_MAJOR_VERSION,__MINGW32_MINOR_VERSION);
