@@ -17,13 +17,13 @@
 %                                 July 2020                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright @ 2020 ImageMagick Studio LLC, a non-profit organization         %
+%  Copyright @ 1999 ImageMagick Studio LLC, a non-profit organization         %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    https://imagemagick.org/script/license.php                               %
+%    https://imagemagick.org/license/                                         %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -53,13 +53,14 @@
 #include "MagickCore/list.h"
 #include "MagickCore/magick.h"
 #include "MagickCore/memory_.h"
+#include "MagickCore/module.h"
 #include "MagickCore/option.h"
 #include "MagickCore/property.h"
 #include "MagickCore/quantum-private.h"
 #include "MagickCore/static.h"
 #include "MagickCore/string_.h"
 #include "MagickCore/string-private.h"
-#include "MagickCore/module.h"
+#include "MagickCore/thread-private.h"
 #include "MagickCore/utility.h"
 #include "MagickCore/xwindow.h"
 #include "MagickCore/xwindow-private.h"
@@ -171,14 +172,12 @@ typedef struct _AshlarInfo
 {
   size_t
     width,
-    height;
+    height,
+    number_nodes;
 
   ssize_t
     align;
 
-  size_t
-    number_nodes;
-
   MagickBooleanType
     best_fit;
 
@@ -231,7 +230,7 @@ static inline ssize_t FindMinimumTileLocation(NodeInfo *first,const ssize_t x,
   y=0;
   extent=0;
   node=first;
-  while (node->x < (ssize_t) (x+width))
+  while (node->x < (x+(ssize_t) width))
   {
     if (node->y > y)
       {
@@ -245,10 +244,10 @@ static inline ssize_t FindMinimumTileLocation(NodeInfo *first,const ssize_t x,
     else
       {
         size_t delta = (size_t) (node->next->x-node->x);
-        if ((delta+extent) > width)
-          delta=width-extent;
-        *excess+=delta*(y-node->y);
-        extent+=delta;
+        if ((delta+(size_t) extent) > width)
+          delta=width-(size_t) extent;
+        *excess+=(ssize_t) delta*(y-node->y);
+        extent+=(ssize_t) delta;
       }
     node=node->next;
   }
@@ -276,8 +275,8 @@ static inline TileInfo AssignBestTileLocation(AshlarInfo *ashlar_info,
     Align along left edge.
   */
   tile.previous=(NodeInfo **) NULL;
-  ashlar_width=(width+ashlar_info->align-1);
-  ashlar_width-=ashlar_width % ashlar_info->align;
+  ashlar_width=(size_t) ((ssize_t) width+ashlar_info->align-1);
+  ashlar_width-=(size_t) ((ssize_t) ashlar_width % ashlar_info->align);
   if ((ashlar_width > ashlar_info->width) || (height > ashlar_info->height))
     {
       /*
@@ -292,7 +291,7 @@ static inline TileInfo AssignBestTileLocation(AshlarInfo *ashlar_info,
   min_excess=(ssize_t) MAGICK_SSIZE_MAX;
   node=ashlar_info->current;
   previous=(&ashlar_info->current);
-  while ((ashlar_width+node->x) <= ashlar_info->width)
+  while (((ssize_t) ashlar_width+node->x) <= (ssize_t) ashlar_info->width)
   {
     ssize_t
       excess,
@@ -309,7 +308,7 @@ static inline TileInfo AssignBestTileLocation(AshlarInfo *ashlar_info,
       }
     else
       {
-        if ((height+y)  <= ashlar_info->height)
+        if (((ssize_t) height+y) <= (ssize_t) ashlar_info->height)
           if ((y < tile.y) || ((y == tile.y) && (excess < min_excess)))
             {
               tile.y=y;
@@ -338,14 +337,14 @@ static inline TileInfo AssignBestTileLocation(AshlarInfo *ashlar_info,
           x,
           y;
 
-        x=tail->x-ashlar_width;
+        x=tail->x-(ssize_t) ashlar_width;
         while (node->next->x <= x)
         {
           previous=(&node->next);
           node=node->next;
         }
         y=FindMinimumTileLocation(node,x,ashlar_width,&excess);
-        if ((height+y) <= ashlar_info->height)
+        if (((ssize_t) height+y) <= (ssize_t) ashlar_info->height)
           {
             if (y <= tile.y)
               if ((y < tile.y) || (excess < min_excess) ||
@@ -389,7 +388,7 @@ static inline TileInfo AssignTileLocation(AshlarInfo *ashlar_info,
    */
    node=ashlar_info->free;
    node->x=(ssize_t) tile.x;
-   node->y=(ssize_t) (tile.y+height);
+   node->y=tile.y+(ssize_t) height;
    ashlar_info->free=node->next;
    /*
      Insert node.
@@ -416,7 +415,7 @@ static inline TileInfo AssignTileLocation(AshlarInfo *ashlar_info,
    }
    node->next=current;
    if (current->x < (tile.x+(ssize_t) width))
-     current->x=(ssize_t) (tile.x+width);
+     current->x=tile.x+(ssize_t) width;
    return(tile);
 }
 
@@ -458,6 +457,9 @@ static inline MagickBooleanType PackAshlarTiles(AshlarInfo *ashlar_info,
   /*
     Pack tiles so they fit the canvas with minimum excess.
   */
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp parallel for schedule(dynamic) shared(status)
+#endif
   for (i=0; i < (ssize_t) number_tiles; i++)
     tiles[i].order=(i);
   qsort((void *) tiles,number_tiles,sizeof(*tiles),CompareTileHeight);
@@ -483,6 +485,9 @@ static inline MagickBooleanType PackAshlarTiles(AshlarInfo *ashlar_info,
   }
   qsort((void *) tiles,number_tiles,sizeof(*tiles),RestoreTileOrder);
   status=MagickTrue;
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp parallel for schedule(dynamic) shared(status)
+#endif
   for (i=0; i < (ssize_t) number_tiles; i++)
   {
     tiles[i].order=(ssize_t) ((tiles[i].x != (ssize_t) MAGICK_SSIZE_MAX) ||
@@ -541,8 +546,8 @@ static Image *ASHLARImage(ImageInfo *image_info,Image *image,
       }
       geometry.width=(size_t) geometry.width/7;
       geometry.height=(size_t) geometry.height/7;
-      geometry.x=(ssize_t) pow((double) geometry.width,0.25);
-      geometry.y=(ssize_t) pow((double) geometry.height,0.25);
+      geometry.x=(ssize_t) pow((double) geometry.width,0.4);
+      geometry.y=(ssize_t) pow((double) geometry.height,0.4);
       image_info->extract=AcquireString("");
       if (image_info->extract != (char *) NULL)
         (void) FormatLocaleString(image_info->extract,MagickPathExtent,
@@ -618,21 +623,32 @@ static Image *ASHLARImage(ImageInfo *image_info,Image *image,
   value=GetImageOption(image_info,"label");
   extent.width=0;
   extent.height=0;
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp parallel for schedule(dynamic) shared(status,extent) \
+    magick_number_threads(image,image,n,1)
+#endif
   for (i=0; i < n; i++)
   {
     Image
       *tile_image;
 
+    if (status == MagickFalse)
+      continue;
     if ((tiles[i].x == (ssize_t) MAGICK_SSIZE_MAX) ||
         (tiles[i].y == (ssize_t) MAGICK_SSIZE_MAX))
       continue;
     tile_image=ResizeImage(GetImageFromList(image,tiles[i].id),(size_t)
-      (tiles[i].width-2*geometry.x),(size_t) (tiles[i].height-2*geometry.y),
-      image->filter,exception);
+      ((ssize_t) tiles[i].width-2*geometry.x),(size_t)
+      ((ssize_t) tiles[i].height-2*geometry.y),image->filter,exception);
     if (tile_image == (Image *) NULL)
-      continue;
-    (void) CompositeImage(ashlar_image,tile_image,image->compose,MagickTrue,
+      {
+        status=MagickFalse;
+        continue;
+      }
+    status=CompositeImage(ashlar_image,tile_image,image->compose,MagickTrue,
       tiles[i].x+geometry.x,tiles[i].y+geometry.y,exception);
+    if (status == MagickFalse)
+      continue;
     if (value != (const char *) NULL)
       {
         char
@@ -647,23 +663,33 @@ static Image *ASHLARImage(ImageInfo *image_info,Image *image,
         if (label != (const char *) NULL)
           {
             (void) CloneString(&draw_info->text,label);
-            draw_info->pointsize=1.8*geometry.y;
+            label=DestroyString(label);
             (void) FormatLocaleString(offset,MagickPathExtent,"%+g%+g",(double)
-              tiles[i].x+geometry.x,(double) tiles[i].height+tiles[i].y+
-              geometry.y/2.0);
+              tiles[i].x+geometry.x,(double) tiles[i].height+tiles[i].y-
+              geometry.y/2.0+4);
             (void) CloneString(&draw_info->geometry,offset);
-            (void) AnnotateImage(ashlar_image,draw_info,exception);
+            status=AnnotateImage(ashlar_image,draw_info,exception);
           }
       }
-    if ((tiles[i].width+tiles[i].x) > extent.width)
-      extent.width=(size_t) (tiles[i].width+tiles[i].x);
-    if ((tiles[i].height+tiles[i].y+geometry.y+2) > extent.height)
-      extent.height=(size_t) (tiles[i].height+tiles[i].y+geometry.y+2);
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+    #pragma omp critical
+#endif
+    {
+      if (((ssize_t) tiles[i].width+tiles[i].x) > (ssize_t) extent.width)
+        extent.width=(size_t) ((ssize_t) tiles[i].width+tiles[i].x);
+      if (((ssize_t) tiles[i].height+tiles[i].y+geometry.y+2) > (ssize_t) extent.height)
+        extent.height=(size_t) ((ssize_t) tiles[i].height+tiles[i].y+
+          geometry.y+2);
+    }
     tile_image=DestroyImage(tile_image);
   }
+  if (image_info->extract != (char *) NULL)
+    (void) ParseAbsoluteGeometry(image_info->extract,&extent);
   (void) SetImageExtent(ashlar_image,extent.width,extent.height,exception);
   nodes=(NodeInfo *) RelinquishMagickMemory(nodes);
   tiles=(CanvasInfo *) RelinquishMagickMemory(tiles);
+  if (status == MagickFalse)
+    ashlar_image=DestroyImage(ashlar_image);
   return(ashlar_image);
 }
 
@@ -706,7 +732,7 @@ static MagickBooleanType WriteASHLARImage(const ImageInfo *image_info,
     tiles_per_page=(size_t) MagickMax(StringToInteger(value),1);
   ashlar_images=NewImageList();
   write_info=CloneImageInfo(image_info);
-  for (i=0; i < (ssize_t) GetImageListLength(image); i+=tiles_per_page)
+  for (i=0; i < (ssize_t) GetImageListLength(image); i+=(ssize_t) tiles_per_page)
   {
     char
       scenes[MagickPathExtent];
@@ -716,7 +742,7 @@ static MagickBooleanType WriteASHLARImage(const ImageInfo *image_info,
       *clone_images;
 
     (void) FormatLocaleString(scenes,MagickPathExtent,"%g-%g",(double) i,
-      (double) (i+tiles_per_page-1));
+      (double) (i+(ssize_t) tiles_per_page-1));
     clone_images=CloneImages(image,scenes,exception);
     if (clone_images == (Image *) NULL)
       {
@@ -744,7 +770,7 @@ static MagickBooleanType WriteASHLARImage(const ImageInfo *image_info,
     GetImageListLength(ashlar_images),exception);
   magick_info=GetMagickInfo(write_info->magick,exception);
   if ((magick_info == (const MagickInfo*) NULL) ||
-      (LocaleCompare(magick_info->module,"ASHLAR") == 0))
+      (LocaleCompare(magick_info->magick_module,"ASHLAR") == 0))
     (void) FormatLocaleString(ashlar_images->filename,MagickPathExtent,
       "miff:%s",write_info->filename);
   status=WriteImages(write_info,ashlar_images,ashlar_images->filename,
