@@ -23,7 +23,7 @@
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    https://imagemagick.org/script/license.php                               %
+%    https://imagemagick.org/license/                                         %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -235,31 +235,25 @@ static Image *ReadPCXImage(const ImageInfo *image_info,ExceptionInfo *exception)
   PCXInfo
     pcx_info;
 
-  ssize_t
-    x;
-
   Quantum
     *q;
 
-  ssize_t
-    i;
-
-  unsigned char
-    *p,
-    *r;
-
   size_t
     one,
     pcx_packets;
 
   ssize_t
     count,
+    i,
+    x,
     y;
 
   unsigned char
+    *p,
     packet,
     pcx_colormap[768],
     *pixels,
+    *r,
     *scanline;
 
   /*
@@ -490,6 +484,8 @@ static Image *ReadPCXImage(const ImageInfo *image_info,ExceptionInfo *exception)
                 /*
                   256 color images have their color map at the end of the file.
                 */
+                offset=SeekBlob(image,(MagickOffsetType) GetBlobSize(image)-3*
+                  image->colors-1,SEEK_SET);
                 pcx_info.colormap_signature=(unsigned char) ReadBlobByte(image);
                 count=ReadBlob(image,3*image->colors,pcx_colormap);
                 p=pcx_colormap;
@@ -541,7 +537,7 @@ static Image *ReadPCXImage(const ImageInfo *image_info,ExceptionInfo *exception)
                 break;
               }
             }
-            r+=pcx_info.planes;
+            r+=(ptrdiff_t) pcx_info.planes;
           }
         }
       else
@@ -640,7 +636,7 @@ static Image *ReadPCXImage(const ImageInfo *image_info,ExceptionInfo *exception)
             if (image->alpha_trait != UndefinedPixelTrait)
               SetPixelAlpha(image,ScaleCharToQuantum(*r++),q);
           }
-        q+=GetPixelChannels(image);
+        q+=(ptrdiff_t) GetPixelChannels(image);
       }
       if (SyncAuthenticPixels(image,exception) == MagickFalse)
         break;
@@ -696,7 +692,8 @@ static Image *ReadPCXImage(const ImageInfo *image_info,ExceptionInfo *exception)
   }
   if (page_table != (MagickOffsetType *) NULL)
     page_table=(MagickOffsetType *) RelinquishMagickMemory(page_table);
-  (void) CloseBlob(image);
+  if (CloseBlob(image) == MagickFalse)
+    status=MagickFalse;
   if (status == MagickFalse)
     return(DestroyImageList(image));
   return(GetFirstImageInList(image));
@@ -809,12 +806,10 @@ static MagickBooleanType PCXWritePixels(PCXInfo *pcx_info,
     *q;
 
   ssize_t
+    count,
     i,
     x;
 
-  ssize_t
-    count;
-
   unsigned char
     packet,
     previous;
@@ -907,6 +902,9 @@ static MagickBooleanType WritePCXImage(const ImageInfo *image_info,Image *image,
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,exception);
   if (status == MagickFalse)
     return(status);
+  number_scenes=GetImageListLength(image);
+  if (number_scenes > MaxNumberScenes)
+    ThrowWriterException(ResourceLimitError,"ListLengthExceedsLimit");
   page_table=(MagickOffsetType *) NULL;
   if ((LocaleCompare(image_info->magick,"DCX") == 0) ||
       ((GetNextImageInList(image) != (Image *) NULL) &&
@@ -916,7 +914,7 @@ static MagickBooleanType WritePCXImage(const ImageInfo *image_info,Image *image,
         Write the DCX page table.
       */
       (void) WriteBlobLSBLong(image,0x3ADE68B1L);
-      page_table=(MagickOffsetType *) AcquireQuantumMemory(MaxNumberScenes,
+      page_table=(MagickOffsetType *) AcquireQuantumMemory(MaxNumberScenes+1,
         sizeof(*page_table));
       if (page_table == (MagickOffsetType *) NULL)
         ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
@@ -924,7 +922,6 @@ static MagickBooleanType WritePCXImage(const ImageInfo *image_info,Image *image,
         (void) WriteBlobLSBLong(image,0x00000000L);
     }
   scene=0;
-  number_scenes=GetImageListLength(image);
   do
   {
     if (page_table != (MagickOffsetType *) NULL)
@@ -952,16 +949,14 @@ static MagickBooleanType WritePCXImage(const ImageInfo *image_info,Image *image,
       case PixelsPerInchResolution:
       default:
       {
-        pcx_info.horizontal_resolution=(unsigned short) image->resolution.x;
-        pcx_info.vertical_resolution=(unsigned short) image->resolution.y;
+        pcx_info.horizontal_resolution=CastDoubleToUShort(image->resolution.x);
+        pcx_info.vertical_resolution=CastDoubleToUShort(image->resolution.y);
         break;
       }
       case PixelsPerCentimeterResolution:
       {
-        pcx_info.horizontal_resolution=(unsigned short)
-          (2.54*image->resolution.x+0.5);
-        pcx_info.vertical_resolution=(unsigned short)
-          (2.54*image->resolution.y+0.5);
+        pcx_info.horizontal_resolution=CastDoubleToUShort(2.54*image->resolution.x+0.5);
+        pcx_info.vertical_resolution=CastDoubleToUShort(2.54*image->resolution.y+0.5);
         break;
       }
     }
@@ -1013,9 +1008,9 @@ static MagickBooleanType WritePCXImage(const ImageInfo *image_info,Image *image,
     if ((image->storage_class == PseudoClass) && (image->colors <= 256))
       for (i=0; i < (ssize_t) image->colors; i++)
       {
-        *q++=ScaleQuantumToChar(image->colormap[i].red);
-        *q++=ScaleQuantumToChar(image->colormap[i].green);
-        *q++=ScaleQuantumToChar(image->colormap[i].blue);
+        *q++=ScaleQuantumToChar((Quantum) image->colormap[i].red);
+        *q++=ScaleQuantumToChar((Quantum) image->colormap[i].green);
+        *q++=ScaleQuantumToChar((Quantum) image->colormap[i].blue);
       }
     (void) WriteBlob(image,3*16,(const unsigned char *) pcx_colormap);
     (void) WriteBlobByte(image,pcx_info.reserved);
@@ -1055,7 +1050,7 @@ static MagickBooleanType WritePCXImage(const ImageInfo *image_info,Image *image,
                 for (x=0; x < (ssize_t) pcx_info.bytes_per_line; x++)
                 {
                   *q++=ScaleQuantumToChar(GetPixelRed(image,p));
-                  p+=GetPixelChannels(image);
+                  p+=(ptrdiff_t) GetPixelChannels(image);
                 }
                 break;
               }
@@ -1064,7 +1059,7 @@ static MagickBooleanType WritePCXImage(const ImageInfo *image_info,Image *image,
                 for (x=0; x < (ssize_t) pcx_info.bytes_per_line; x++)
                 {
                   *q++=ScaleQuantumToChar(GetPixelGreen(image,p));
-                  p+=GetPixelChannels(image);
+                  p+=(ptrdiff_t) GetPixelChannels(image);
                 }
                 break;
               }
@@ -1073,7 +1068,7 @@ static MagickBooleanType WritePCXImage(const ImageInfo *image_info,Image *image,
                 for (x=0; x < (ssize_t) pcx_info.bytes_per_line; x++)
                 {
                   *q++=ScaleQuantumToChar(GetPixelBlue(image,p));
-                  p+=GetPixelChannels(image);
+                  p+=(ptrdiff_t) GetPixelChannels(image);
                 }
                 break;
               }
@@ -1083,7 +1078,7 @@ static MagickBooleanType WritePCXImage(const ImageInfo *image_info,Image *image,
                 for (x=(ssize_t) pcx_info.bytes_per_line; x != 0; x--)
                 {
                   *q++=ScaleQuantumToChar((Quantum) (GetPixelAlpha(image,p)));
-                  p+=GetPixelChannels(image);
+                  p+=(ptrdiff_t) GetPixelChannels(image);
                 }
                 break;
               }
@@ -1112,7 +1107,7 @@ static MagickBooleanType WritePCXImage(const ImageInfo *image_info,Image *image,
             for (x=0; x < (ssize_t) image->columns; x++)
             {
               *q++=(unsigned char) ((ssize_t) GetPixelIndex(image,p));
-              p+=GetPixelChannels(image);
+              p+=(ptrdiff_t) GetPixelChannels(image);
             }
             if (PCXWritePixels(&pcx_info,pixels,image) == MagickFalse)
               break;
@@ -1144,7 +1139,7 @@ static MagickBooleanType WritePCXImage(const ImageInfo *image_info,Image *image,
               for (x=0; x < (ssize_t) image->columns; x++)
               {
                 byte<<=1;
-                if (GetPixelLuma(image,p) < (QuantumRange/2.0))
+                if (GetPixelLuma(image,p) < ((double) QuantumRange/2.0))
                   byte|=0x01;
                 bit++;
                 if (bit == 8)
@@ -1153,7 +1148,7 @@ static MagickBooleanType WritePCXImage(const ImageInfo *image_info,Image *image,
                     bit=0;
                     byte=0;
                   }
-                p+=GetPixelChannels(image);
+                p+=(ptrdiff_t) GetPixelChannels(image);
               }
               if (bit != 0)
                 *q++=byte << (8-bit);
@@ -1175,8 +1170,6 @@ static MagickBooleanType WritePCXImage(const ImageInfo *image_info,Image *image,
     pcx_colormap=(unsigned char *) RelinquishMagickMemory(pcx_colormap);
     if (page_table == (MagickOffsetType *) NULL)
       break;
-    if (scene >= 1023)
-      break;
     if (GetNextImageInList(image) == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
@@ -1211,6 +1204,7 @@ static MagickBooleanType WritePCXImage(const ImageInfo *image_info,Image *image,
         "UnableToWriteFile","`%s': %s",image->filename,message);
       message=DestroyString(message);
     }
-  (void) CloseBlob(image);
-  return(MagickTrue);
+  if (CloseBlob(image) == MagickFalse)
+    status=MagickFalse;
+  return(status);
 }
