@@ -17,13 +17,13 @@
 %                                   May 2019                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright @ 2019 ImageMagick Studio LLC, a non-profit organization         %
+%  Copyright @ 1999 ImageMagick Studio LLC, a non-profit organization         %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    https://imagemagick.org/script/license.php                               %
+%    https://imagemagick.org/license/                                         %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -75,9 +75,6 @@ typedef struct _TIM2FileHeader
 
   unsigned short
     image_count;
-
-  char
-    reserved[8];
 } TIM2FileHeader;
 
 typedef struct _TIM2ImageHeader
@@ -245,9 +242,6 @@ static MagickBooleanType ReadTIM2ImageData(const ImageInfo *image_info,
   Quantum
     *q;
 
-  unsigned char
-    *p;
-
   size_t
     bits_per_line,
     bytes_per_line;
@@ -257,6 +251,7 @@ static MagickBooleanType ReadTIM2ImageData(const ImageInfo *image_info,
     y;
 
   unsigned char
+    *p,
     *row_data;
 
   unsigned int
@@ -274,7 +269,7 @@ static MagickBooleanType ReadTIM2ImageData(const ImageInfo *image_info,
   /*
    * Image data
    */
-  bits_per_line=image->columns*bits_per_pixel;
+  bits_per_line=image->columns*(size_t) bits_per_pixel;
   bytes_per_line=bits_per_line/8 + ((bits_per_line%8==0) ? 0 : 1);
   row_data=(unsigned char*) AcquireQuantumMemory(1,bytes_per_line);
   if (row_data == (unsigned char *) NULL)
@@ -308,17 +303,17 @@ static MagickBooleanType ReadTIM2ImageData(const ImageInfo *image_info,
             p=row_data;
             for (x=0; x < ((ssize_t) image->columns-1); x+=2)
             {
-              SetPixelIndex(image,(*p >> 0) & 0x0F,q);
-              q+=GetPixelChannels(image);
-              SetPixelIndex(image,(*p >> 4) & 0x0F,q);
+              SetPixelIndex(image,(Quantum)((*p >> 0) & 0x0f),q);
+              q+=(ptrdiff_t) GetPixelChannels(image);
+              SetPixelIndex(image,(Quantum)((*p >> 4) & 0x0f),q);
               p++;
-              q+=GetPixelChannels(image);
+              q+=(ptrdiff_t) GetPixelChannels(image);
             }
             if ((image->columns % 2) != 0)
               {
-                SetPixelIndex(image,(*p >> 4) & 0x0F,q);
+                SetPixelIndex(image,(Quantum)((*p >> 4) & 0x0f),q);
                 p++;
-                q+=GetPixelChannels(image);
+                q+=(ptrdiff_t) GetPixelChannels(image);
               }
             if (SyncAuthenticPixels(image,exception) == MagickFalse)
               break;
@@ -351,7 +346,7 @@ static MagickBooleanType ReadTIM2ImageData(const ImageInfo *image_info,
             {
               SetPixelIndex(image,*p,q);
               p++;
-              q+=GetPixelChannels(image);
+              q+=(ptrdiff_t) GetPixelChannels(image);
             }
             if (SyncAuthenticPixels(image,exception) == MagickFalse)
               break;
@@ -372,6 +367,7 @@ static MagickBooleanType ReadTIM2ImageData(const ImageInfo *image_info,
             image_info->filename);
         }
       }
+      SyncImage(image,exception);
     }
   else  /* has_clut==false */
     {
@@ -401,8 +397,8 @@ static MagickBooleanType ReadTIM2ImageData(const ImageInfo *image_info,
               SetPixelGreen(image,GetChannelValue(word,1,RGBA16),q);
               SetPixelBlue(image,GetChannelValue(word,2,RGBA16),q);
               SetPixelAlpha(image,GetAlpha(word,RGBA16),q);
-              q+=GetPixelChannels(image);
-              p+=sizeof(unsigned short);
+              q+=(ptrdiff_t) GetPixelChannels(image);
+              p+=(ptrdiff_t) sizeof(unsigned short);
             }
             if (SyncAuthenticPixels(image,exception) == MagickFalse)
               break;
@@ -440,8 +436,8 @@ static MagickBooleanType ReadTIM2ImageData(const ImageInfo *image_info,
               SetPixelRed(image,GetChannelValue(word,0,RGB24),q);
               SetPixelGreen(image,GetChannelValue(word,1,RGB24),q);
               SetPixelBlue(image,GetChannelValue(word,2,RGB24),q);
-              q+=GetPixelChannels(image);
-              p+=3;
+              q+=(ptrdiff_t) GetPixelChannels(image);
+              p+=(ptrdiff_t) 3;
             }
             if (SyncAuthenticPixels(image,exception) == MagickFalse)
               break;
@@ -481,8 +477,8 @@ static MagickBooleanType ReadTIM2ImageData(const ImageInfo *image_info,
               SetPixelGreen(image,GetChannelValue(word,1,RGBA32),q);
               SetPixelBlue(image,GetChannelValue(word,2,RGBA32),q);
               SetPixelAlpha(image,GetAlpha(word,RGBA32),q);
-              q+=GetPixelChannels(image);
-              p+=4;
+              q+=(ptrdiff_t) GetPixelChannels(image);
+              p+=(ptrdiff_t) 4;
             }
             if (SyncAuthenticPixels(image,exception) == MagickFalse)
               break;
@@ -510,6 +506,9 @@ static MagickBooleanType ReadTIM2ImageData(const ImageInfo *image_info,
     CSM
       csm;
 
+    size_t
+      clut_size;
+
     ssize_t
       i;
 
@@ -519,12 +518,16 @@ static MagickBooleanType ReadTIM2ImageData(const ImageInfo *image_info,
     /*
       * ### Read CLUT Data ###
       */
-    clut_data=(unsigned char *) AcquireQuantumMemory(1,header->clut_size);
+    clut_size=MagickMax(header->clut_size,(size_t) (clut_depth/8)*
+      image->colors);
+    clut_data=(unsigned char *) AcquireQuantumMemory(clut_size,
+      sizeof(*clut_data));
     if (clut_data == (unsigned char *) NULL)
       ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
         image_info->filename);
-    count=ReadBlob(image,header->clut_size,clut_data);
-    if (count != (ssize_t) (header->clut_size))
+    (void) memset(clut_data,0,clut_size);
+    count=ReadBlob(image,clut_size,clut_data);
+    if (count != (ssize_t) clut_size)
       {
         clut_data=(unsigned char *) RelinquishMagickMemory(clut_data);
         ThrowBinaryException(CorruptImageError,"InsufficientImageDataInFile",
@@ -540,14 +543,14 @@ static MagickBooleanType ReadTIM2ImageData(const ImageInfo *image_info,
       {
         for (i=0; i < (ssize_t) image->colors; i++)
         {
-          word = ((unsigned short)* p   )<<0*8 |
-                  ((unsigned short)*(p+1))<<1*8;
+          word = (unsigned int) (((unsigned short)* p   )<<0*8 |
+                  ((unsigned short)*(p+1))<<1*8);
 
           image->colormap[i].red=GetChannelValue(word,0,RGBA16);
           image->colormap[i].green=GetChannelValue(word,1,RGBA16);
           image->colormap[i].blue=GetChannelValue(word,2,RGBA16);
           image->colormap[i].alpha=GetAlpha(word,RGBA16);
-          p+=2;
+          p+=(ptrdiff_t) 2;
         }
         break;
       }
@@ -562,7 +565,7 @@ static MagickBooleanType ReadTIM2ImageData(const ImageInfo *image_info,
           image->colormap[i].red=GetChannelValue(word,0,RGB24);
           image->colormap[i].green=GetChannelValue(word,1,RGB24);
           image->colormap[i].blue=GetChannelValue(word,2,RGB24);
-          p+=3;
+          p+=(ptrdiff_t) 3;
         }
         break;
       }
@@ -579,7 +582,7 @@ static MagickBooleanType ReadTIM2ImageData(const ImageInfo *image_info,
           image->colormap[i].green=GetChannelValue(word,1,RGBA32);
           image->colormap[i].blue=GetChannelValue(word,2,RGBA32);
           image->colormap[i].alpha=GetAlpha(word,RGBA32);
-          p+=4;
+          p+=(ptrdiff_t) 4;
         }
         break;
       }
@@ -626,8 +629,7 @@ static Image *ReadTIM2Image(const ImageInfo *image_info,
     status;
 
   ssize_t
-    i,
-    str_read;
+    i;
 
   TIM2FileHeader
     file_header;
@@ -637,12 +639,11 @@ static Image *ReadTIM2Image(const ImageInfo *image_info,
    */
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickCoreSignature);
-
-  if (image_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
-      image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickCoreSignature);
+  if (IsEventLogging() != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      image_info->filename);
   image=AcquireImage(image_info,exception);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
@@ -650,37 +651,19 @@ static Image *ReadTIM2Image(const ImageInfo *image_info,
       image=DestroyImageList(image);
       return((Image *) NULL);
     }
-  /*
-   * Verify TIM2 magic number.
-   */
   file_header.magic_num=ReadBlobMSBLong(image);
   if (file_header.magic_num != 0x54494D32) /* "TIM2" */
     ThrowReaderException(CorruptImageError,"ImproperImageHeader");
-  /*
-   * #### Read File Header ####
-   */
-  file_header.format_vers=ReadBlobByte(image);
+  file_header.format_vers=(unsigned char) ReadBlobByte(image);
   if (file_header.format_vers != 0x04)
     ThrowReaderException(CoderError,"ImageTypeNotSupported");
-  file_header.format_type=ReadBlobByte(image);
+  file_header.format_type=(unsigned char) ReadBlobByte(image);
   file_header.image_count=ReadBlobLSBShort(image);
-  ReadBlobStream(image,8,&(file_header.reserved),&str_read);
-  /*
-   * Jump to first image header
-   */
-  switch(file_header.format_type)
-  {
-    case 0x00:
-      if (DiscardBlobBytes(image,16) == MagickFalse)
-        ThrowReaderException(CorruptImageError,"InsufficientImageDataInFile");
-      break;
-    case 0x01:
-      if (DiscardBlobBytes(image,128) == MagickFalse)
-        ThrowReaderException(CorruptImageError,"InsufficientImageDataInFile");
-      break;
-    default:
-      ThrowReaderException(CoderError,"ImageTypeNotSupported");
-  }
+  if (DiscardBlobBytes(image,8) == MagickFalse) /* reserved */
+    ThrowReaderException(CorruptImageError,"InsufficientImageDataInFile");
+  if ((file_header.format_type > 0) &&
+      (DiscardBlobBytes(image,112) == MagickFalse))
+    ThrowReaderException(CorruptImageError,"InsufficientImageDataInFile");
   /*
    * Process each image. Only one image supported for now
    */
@@ -695,6 +678,29 @@ static Image *ReadTIM2Image(const ImageInfo *image_info,
     TIM2ImageHeader
       image_header;
 
+    if (i > 0)
+      {
+        /*
+          Proceed to next image.
+        */
+        if (image_info->number_scenes != 0)
+          if (image->scene >= (image_info->scene+image_info->number_scenes-1))
+            break;
+        /*
+          Allocate next image structure.
+        */
+        AcquireNextImage(image_info,image,exception);
+        if (GetNextImageInList(image) == (Image *) NULL)
+          {
+            status=MagickFalse;
+            break;
+          }
+        image=SyncNextImageInList(image);
+        status=SetImageProgress(image,LoadImagesTag,(MagickOffsetType)
+          image->scene-1,image->scene);
+        if (status == MagickFalse)
+          break;
+      }
     ReadTIM2ImageHeader(image,&image_header);
     if (image_header.mipmap_count != 1)
       ThrowReaderException(CoderError,"NumberOfImagesIsNotSupported");
@@ -746,10 +752,10 @@ static Image *ReadTIM2Image(const ImageInfo *image_info,
         ThrowReaderException(CorruptImageError,"ImproperImageHeader");
         break;
     }
-    image->depth=(clut_depth != 0) ? clut_depth : bits_per_pixel;
+    image->depth=(size_t) ((clut_depth != 0) ? clut_depth : bits_per_pixel);
     if ((image->depth == 16) || (image->depth == 32))
       image->alpha_trait=BlendPixelTrait;
-    if (image->ping != MagickFalse)
+    if (image->ping == MagickFalse)
       {
         status=ReadTIM2ImageData(image_info,image,&image_header,clut_depth,
           bits_per_pixel,exception);
@@ -765,27 +771,9 @@ static Image *ReadTIM2Image(const ImageInfo *image_info,
           image->filename);
         break;
       }
-    /*
-      Proceed to next image.
-    */
-    if (image_info->number_scenes != 0)
-      if (image->scene >= (image_info->scene+image_info->number_scenes-1))
-        break;
-    /*
-      Allocate next image structure.
-    */
-    AcquireNextImage(image_info,image,exception);
-    if (GetNextImageInList(image) == (Image *) NULL)
-      {
-        status=MagickFalse;
-        break;
-      }
-    image=SyncNextImageInList(image);
-    status=SetImageProgress(image,LoadImagesTag,image->scene-1,image->scene);
-    if (status == MagickFalse)
-      break;
   }
-  (void) CloseBlob(image);
+  if (CloseBlob(image) == MagickFalse)
+    status=MagickFalse;
   if (status == MagickFalse)
     return(DestroyImageList(image));
   return(GetFirstImageInList(image));
